"""..."""
from dataclasses import dataclass

from aligo.types import *
from aligo.types.Enum import *


@dataclass
class GetShareLinkListRequest(DataClass):
    """获取自己的分享列表

    Attributes:
        creator (str): 一 、管理员：1.1 不传，查所有用户 1.2 传指定user_id，查指定用户的分享。
            * 二、普通用户：2.1 不传，查自己2.2 传指定user_id，只能是自己，否则报403
        limit (int): 返回数据最大条数,范围:[1-100]，默认:100
        marker (str):
        order_by (GetShareLinkListOrderBy): 排序字段,默认按照创建时间倒序输出。
        order_direction (str):
        include_canceled (bool): 列举结果是否包含已经取消的分享
    """
    creator: str = None
    limit: int = 100
    marker: str = ''
    order_by: GetShareLinkListOrderBy = 'created_at'
    order_direction: OrderDirection = 'DESC'
    include_canceled: bool = False



"""..."""
from dataclasses import dataclass, field
from typing import List

from .BaseFile import BaseFile
from .DataClass import DataClass


@dataclass
class _BaseAlbumList(DataClass):
    """..."""
    list: List[BaseFile] = field(default_factory=list)


@dataclass
class BaseAlbum(DataClass):
    """..."""
    owner: str = None
    name: str = None
    description: str = None
    album_id: str = None
    file_count: int = None
    image_count: int = None
    video_count: int = None
    created_at: int = None
    updated_at: int = None
    cover: _BaseAlbumList = None



"""GetFilePathResponse"""
from dataclasses import dataclass, field
from typing import List

from aligo.types import *


@dataclass
class GetFilePathResponse(DataClass):
    """GetFilePathResponse"""
    items: List[BaseFile] = field(default_factory=list)



"""..."""
from __future__ import annotations

import logging
import os
from dataclasses import dataclass, is_dataclass
from typing import TypeVar, Generic, Optional, List, Dict, Type

import coloredlogs
from typing_extensions import get_type_hints, get_origin, get_args

DataType = TypeVar('DataType')

_HINTS = {}
_LOGGER = logging.getLogger(__name__)
_ALIGO_DEBUG = os.getenv('ALIGO_DEBUG')

coloredlogs.install(
    level=logging.DEBUG,
    logger=_LOGGER,
    milliseconds=True,
    datefmt='%X',
    fmt=f'MISSING_ATTRS.%(levelname)s %(message)s'
)


@dataclass
class DataClass:
    """..."""

    @staticmethod
    def _get_hints(cls: Type[DataClass]):
        """..."""
        hints = _HINTS.get(cls)
        if hints:
            return hints
        hints = get_type_hints(cls)
        _HINTS[cls] = hints
        return hints

    @staticmethod
    def fill_attrs(cls: Type[DataType], obj: Dict) -> DataType:
        """..."""
        hints = DataClass._get_hints(cls)
        params = {}
        for key, value in obj.items():
            if key in hints:
                params[key] = value
            else:
                if _ALIGO_DEBUG:
                    _LOGGER.warning(
                        f'{cls.__module__}.{cls.__name__}({key} : {type(value).__name__} = {repr(value)[:100]})')
        return cls(**params)

    def __post_init__(self):
        """序列化属性"""
        hints = DataClass._get_hints(self.__class__)
        for k, v in hints.items():
            origin = get_origin(v)
            if origin is None:
                if is_dataclass(v):
                    setattr(self, k, _null_dict(v, getattr(self, k)))
                    continue
            for hint_type in get_args(v):
                if is_dataclass(hint_type):
                    if origin is list:
                        setattr(self, k, _null_list(hint_type, getattr(self, k)))
                    break


def _null_list(cls: Generic[DataType], may_null: Optional[List[DataType]]) -> List[DataType]:
    if may_null and len(may_null) != 0:
        if isinstance(may_null[0], dict):
            # noinspection PyProtectedMember
            return [DataClass.fill_attrs(cls, i) for i in may_null]
        else:
            return may_null
    else:
        return []


def _null_dict(cls: Type[DataType], may_null: Optional[Dict]) -> DataType:
    if may_null is None:
        return None
    if isinstance(may_null, dict):
        # noinspection PyProtectedMember
        return DataClass.fill_attrs(cls, may_null)
    return may_null



"""..."""
from dataclasses import dataclass

from .DataClass import DataClass


@dataclass
class AudioMusicMeta(DataClass):
    """..."""
    title: str = None
    artist: str = None
    album: str = None
    cover_url: str = None



"""..."""
from dataclasses import dataclass

from aligo.types import DataClass


@dataclass
class SearchShareFileRequest(DataClass):
    """..."""
    share_id: str = None
    keyword: str = None
    order_by: str = None
    limit: int = 100
    marker: str = None



"""..."""
from dataclasses import dataclass
from typing import List

from aligo.types import DataClass


@dataclass
class CreateShareLinkRequest(DataClass):
    """创建分享链接请求

    Attributes:
        drive_id (str):
        file_id_list (List[str]):
        share_pwd (str): 提取码，0-64个字符。长度0表示没有提取码。
        expiration (str): 失效时间点。RFC3339格式，比如："2020-06-28T11:33:00.000+08:00"。永久有效: ""
        description (str): 描述
        share_name (str): 分享名，默认使用第一个文件名
    """
    drive_id: str = None
    file_id_list: List[str] = None
    share_pwd: str = None
    expiration: str = ''
    description: str = None
    share_name: str = None



"""..."""
from dataclasses import dataclass, field

from .DataClass import DataClass


@dataclass
class UserConfig(DataClass):
    """
    {
      "id": 601978,
      "face_ai_recognition_status": 1,
      "auto_generate_memory": 0,
      "is_sfiia_entry_hidden": false,
      "homepage_visibility": 1,
      "folder_cover_enable": false,
      "auto_generate_photo_highlights": true,
      "auto_generate_photo_scene": true,
      "file_resume": false,
      "personal_recommendation": true,
      "gmt_create": "2021-05-30T12:59:45.000+00:00",
      "gmt_modified": "2021-05-30T12:59:45.000+00:00",
      "user_id": "2a9348a769594ca45acc7c140945678f"
    }
    """
    id: int = field(default=None)
    face_ai_recognition_status: int = field(default=None)
    auto_generate_memory: int = field(default=None)
    is_sfiia_entry_hidden: bool = field(default=None)
    homepage_visibility: int = field(default=None)
    folder_cover_enable: bool = field(default=None)
    auto_generate_photo_highlights: bool = field(default=None)
    auto_generate_photo_scene: bool = field(default=None)
    file_resume: bool = field(default=None)
    personal_recommendation: bool = field(default=None)
    gmt_create: str = field(default=None)
    gmt_modified: str = field(default=None)
    user_id: str = field(default=None)



"""字面量值"""
from typing import Optional

from typing_extensions import Literal

MediaTranscodeStatus = Optional[
    # 媒体转码状态
    Literal[
        'running',  # 转码中
        'finished',  # 转码完成
        'failed',  # 转码失败
    ]
]

GetFileListOrderBy = Optional[
    # 获取文件列表排序方式
    Literal[
        'name',  # 文件名
        'created_at',  # 创建时间
        'updated_at',  # 更新时间
        'size',  # 文件大小
    ]
]

SearchFileOrderBy = Optional[
    # 搜索文件排序方式
    Literal[
        'name',  # 文件名
        'created_at',  # 创建时间
        'updated_at',  # 更新时间
        'size',  # 文件大小
    ]
]

GetFileListFields = Optional[
    # 获取文件列表字段
    Literal[
        '*',  # 全部字段
        'thumbnail',  # 缩略图
    ]
]

GetFileFields = GetFileListFields

BaseFileCategory = Optional[
    # 文件类型
    Literal[
        'others',  # 其他
        'doc',  # 文档
        'image',  # 图片
        'audio',  # 音频
        'video',  # 视频
    ]
]

OrderDirection = Optional[
    # 排序方向
    Literal[
        'ASC',  # 升序
        'DESC',  # 降序
    ]
]

BaseFileType = Optional[
    # 文件类型
    Literal[
        'file',  # 文件
        'folder',  # 文件夹
    ]
]

CheckNameMode = Optional[
    # 名称检查模式
    Literal[
        'auto_rename',  # 自动重命名
        'refuse',  # 拒绝
        'overwrite',  # 覆盖
    ]
]

BaseFileContentHashName = Optional[
    # 文件内容哈希名称
    Literal[
        'sha1'
    ]
]

BaseFileStatus = Optional[
    # 文件状态
    Literal[
        'uploading',  # 上传中
        'available',  # 可用
    ]
]

GetRecycleBinListOrderBy = Optional[
    # 获取回收站文件列表排序方式
    Literal[
        'name',  # 文件名
    ]
]

SharePolicy = Optional[
    # 分享策略
    Literal[
        'url',  # 分享链接
        'msg',  # 分享消息
    ]
]

GetShareFileListOrderBy = Optional[
    # 获取分享文件列表排序方式
    Literal[
        'name',  # 文件名
        'updated_at',  # 更新时间
    ]
]

SearchCategory = Optional[
    # 搜索类型
    Literal[
        'image',  # 图片
        'video',  # 视频
        'audio',  # 音频
        'app',  # 应用
        'doc',  # 文档
        'others',  # 其他
    ]
]

GetShareLinkListOrderBy = Optional[
    # 获取分享链接列表排序方式
    Literal[
        'share_name',  # 分享名称
        'created_at',  # 创建时间
        'description',  # 描述
        'updated_at',  # 更新时间
    ]
]

GetStarredListFields = Optional[
    # 获取收藏文件列表字段
    Literal[
        '*',  # 全部字段
        'thumbnail',  # 缩略图
    ]
]

VideoTemplateID = Optional[
    # 视频模板ID
    Literal[
        '',  # 默认模板
        'FHD',  # 全高清
        'HD',  # 高清
        'SD',  # 标清
        'LD',  # 普清
    ]
]

ArchiveType = Literal[
    "zip", "rar", "rar5", "rar4"
]

GetVideoPreviewCategory = Optional[
    # 获取视频预览类型
    Literal[
        'live_transcoding',  # 在线直播转码（m3u8）
    ]
]

# 相册文件排序方式
AlbumFileListType = Literal[
    "joined_at"
]

# 模板
TemplateType = Literal[
    1, 2, 3
]



"""..."""
from typing import Iterator

from aligo.core import *
from aligo.core.Config import *
from aligo.request import *
from aligo.response import *
from aligo.types import *


class File(BaseAligo):
    """..."""

    def _core_get_file_list(self, body: GetFileListRequest) -> Iterator[BaseFile]:
        """..."""
        yield from self._list_file(ADRIVE_V3_FILE_LIST, body, GetFileListResponse)

    def _core_batch_get_files(self, body: BatchGetFileRequest) -> Iterator[BatchSubResponse]:
        """batch_get_files"""
        if body.drive_id is None:
            body.drive_id = self.default_drive_id

        yield from self.batch_request(BatchRequest(
            requests=[BatchSubRequest(
                id=file_id,
                url='/file/get',
                body=GetFileRequest(
                    drive_id=body.drive_id, file_id=file_id
                )
            ) for file_id in body.file_id_list]
        ), GetFileRequest)



"""分享链接下载链接请求体"""
from dataclasses import dataclass

from aligo.types import *


@dataclass
class GetShareLinkDownloadUrlRequest(DataClass):
    """..."""
    share_id: str
    file_id: str
    # {"code":"InvalidParameter.ExpireSec","message":"The input parameter expire_sec is not valid. expire_sec should be less or equal than 600"}
    # expire_sec: int = 14400
    expire_sec: int = 600
    image_thumbnail_process: str = None
    image_url_process: str = None
    video_thumbnail_process: str = None
    get_video_play_info: bool = None
    get_audio_play_info: bool = None



"""..."""
from dataclasses import dataclass

from aligo.types import DataClass
from aligo.types.Enum import ArchiveType

@dataclass
class ArchiveUncompressRequest(DataClass):
    """..."""
    file_id: str = None
    target_file_id: str = 'root'
    archive_type: ArchiveType = 'zip'
    drive_id: str = None
    target_drive_id: str = None
    domain_id: str = ''



"""..."""

from aligo.core import *
from aligo.core.Config import *
from aligo.request import *
from aligo.response import *


class Audio(BaseAligo):
    """..."""

    def _core_get_audio_play_info(self, body: GetAudioPlayInfoRequest) -> GetAudioPlayInfoResponse:
        """获取音频播放信息"""
        response = self._post(V2_DATABOX_GET_AUDIO_PLAY_INFO, body=body)
        return self._result(response, GetAudioPlayInfoResponse)



"""..."""
from dataclasses import dataclass, field
from typing import List

from aligo.types import DataClass


@dataclass
class BatchShareFileSaveToDriveRequest(DataClass):
    """..."""
    share_id: str
    file_id_list: List[str] = field(default_factory=list)
    to_parent_file_id: str = 'root'
    auto_rename: bool = True
    to_drive_id: str = None
